# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/ids_finder',
                'doc_host': 'https://Beforerr.github.io',
                'git_url': 'https://github.com/Beforerr/ids_finder',
                'lib_path': 'discontinuitypy'},
  'syms': { 'discontinuitypy.config': { 'discontinuitypy.config.IDsConfig': ('ids_config.html#idsconfig', 'discontinuitypy/config.py'),
                                        'discontinuitypy.config.IDsConfig.export': ( 'ids_config.html#idsconfig.export',
                                                                                     'discontinuitypy/config.py'),
                                        'discontinuitypy.config.IDsConfig.fname': ( 'ids_config.html#idsconfig.fname',
                                                                                    'discontinuitypy/config.py'),
                                        'discontinuitypy.config.IDsConfig.load': ( 'ids_config.html#idsconfig.load',
                                                                                   'discontinuitypy/config.py'),
                                        'discontinuitypy.config.IDsConfig.path': ( 'ids_config.html#idsconfig.path',
                                                                                   'discontinuitypy/config.py'),
                                        'discontinuitypy.config.IDsConfig.timeranges': ( 'ids_config.html#idsconfig.timeranges',
                                                                                         'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig': ( 'ids_config.html#speasyidsconfig',
                                                                                    'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig._get_and_process_data': ( 'ids_config.html#speasyidsconfig._get_and_process_data',
                                                                                                          'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.e_temp_var': ( 'ids_config.html#speasyidsconfig.e_temp_var',
                                                                                               'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.get_and_process_data': ( 'ids_config.html#speasyidsconfig.get_and_process_data',
                                                                                                         'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.get_vars': ( 'ids_config.html#speasyidsconfig.get_vars',
                                                                                             'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.get_vars_df': ( 'ids_config.html#speasyidsconfig.get_vars_df',
                                                                                                'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.ion_temp_var': ( 'ids_config.html#speasyidsconfig.ion_temp_var',
                                                                                                 'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.mag_vars': ( 'ids_config.html#speasyidsconfig.mag_vars',
                                                                                             'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.model_post_init': ( 'ids_config.html#speasyidsconfig.model_post_init',
                                                                                                    'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.plasma_vars': ( 'ids_config.html#speasyidsconfig.plasma_vars',
                                                                                                'discontinuitypy/config.py'),
                                        'discontinuitypy.config.SpeasyIDsConfig.set_data_from_vars': ( 'ids_config.html#speasyidsconfig.set_data_from_vars',
                                                                                                       'discontinuitypy/config.py'),
                                        'discontinuitypy.config.standardize_plasma_data': ( 'ids_config.html#standardize_plasma_data',
                                                                                            'discontinuitypy/config.py')},
            'discontinuitypy.core.detection': { 'discontinuitypy.core.detection.detect_events': ( 'ids_detection.html#detect_events',
                                                                                                  'discontinuitypy/core/detection.py'),
                                                'discontinuitypy.core.detection.pl_format_time': ( 'ids_detection.html#pl_format_time',
                                                                                                   'discontinuitypy/core/detection.py')},
            'discontinuitypy.core.event': { 'discontinuitypy.core.event.get_event_data': ( 'event.html#get_event_data',
                                                                                           'discontinuitypy/core/event.py')},
            'discontinuitypy.core.pipeline': { 'discontinuitypy.core.pipeline.compress_data_by_events': ( 'ids_finder.html#compress_data_by_events',
                                                                                                          'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.compress_data_by_intervals': ( 'ids_finder.html#compress_data_by_intervals',
                                                                                                             'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.compress_dfs_by_intervals': ( 'ids_finder.html#compress_dfs_by_intervals',
                                                                                                            'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.extract_features': ( 'ids_finder.html#extract_features',
                                                                                                   'discontinuitypy/core/pipeline.py'),
                                               'discontinuitypy.core.pipeline.ids_finder': ( 'ids_finder.html#ids_finder',
                                                                                             'discontinuitypy/core/pipeline.py')},
            'discontinuitypy.core.propeties': { 'discontinuitypy.core.propeties.IDsPdPipeline': ( 'ids_properties.html#idspdpipeline',
                                                                                                  'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPdPipeline.calc_duration': ( 'ids_properties.html#idspdpipeline.calc_duration',
                                                                                                                'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPdPipeline.calc_mva_features': ( 'ids_properties.html#idspdpipeline.calc_mva_features',
                                                                                                                    'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPdPipeline.calc_normal_direction': ( 'ids_properties.html#idspdpipeline.calc_normal_direction',
                                                                                                                        'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPdPipeline.calc_rotation_angle': ( 'ids_properties.html#idspdpipeline.calc_rotation_angle',
                                                                                                                      'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.IDsPdPipeline.calc_vec_change': ( 'ids_properties.html#idspdpipeline.calc_vec_change',
                                                                                                                  'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_candidate_duration': ( 'ids_properties.html#calc_candidate_duration',
                                                                                                            'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_normal_direction': ( 'ids_properties.html#calc_events_normal_direction',
                                                                                                                 'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_rotation_angle': ( 'ids_properties.html#calc_events_rotation_angle',
                                                                                                               'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_events_vec_change': ( 'ids_properties.html#calc_events_vec_change',
                                                                                                           'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_normal_direction': ( 'ids_properties.html#calc_normal_direction',
                                                                                                          'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.calc_rotation_angle': ( 'ids_properties.html#calc_rotation_angle',
                                                                                                        'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_candidate_data': ( 'ids_properties.html#get_candidate_data',
                                                                                                       'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_candidates': ( 'ids_properties.html#get_candidates',
                                                                                                   'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.get_data_at_times': ( 'ids_properties.html#get_data_at_times',
                                                                                                      'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.pdp.ApplyToRows._transform': ( 'ids_properties.html#pdp.applytorows._transform',
                                                                                                               'discontinuitypy/core/propeties.py'),
                                                'discontinuitypy.core.propeties.process_events': ( 'ids_properties.html#process_events',
                                                                                                   'discontinuitypy/core/propeties.py')},
            'discontinuitypy.datasets': { 'discontinuitypy.datasets.IDsDataset': ( 'datasets.html#idsdataset',
                                                                                   'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.plot': ( 'datasets.html#idsdataset.plot',
                                                                                        'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.plot_candidate': ( 'datasets.html#idsdataset.plot_candidate',
                                                                                                  'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.plot_candidates': ( 'datasets.html#idsdataset.plot_candidates',
                                                                                                   'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.update_events': ( 'datasets.html#idsdataset.update_events',
                                                                                                 'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.update_events_with_plasma_data': ( 'datasets.html#idsdataset.update_events_with_plasma_data',
                                                                                                                  'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IDsDataset.update_events_with_temp_data': ( 'datasets.html#idsdataset.update_events_with_temp_data',
                                                                                                                'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents': ('datasets.html#idsevents', 'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.Config': ( 'datasets.html#idsevents.config',
                                                                                         'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.export': ( 'datasets.html#idsevents.export',
                                                                                         'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.find_events': ( 'datasets.html#idsevents.find_events',
                                                                                              'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.get_event': ( 'datasets.html#idsevents.get_event',
                                                                                            'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.get_event_data': ( 'datasets.html#idsevents.get_event_data',
                                                                                                 'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.IdsEvents.get_events': ( 'datasets.html#idsevents.get_events',
                                                                                             'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.log_event_change': ( 'datasets.html#log_event_change',
                                                                                         'discontinuitypy/datasets.py'),
                                          'discontinuitypy.datasets.write': ('datasets.html#write', 'discontinuitypy/datasets.py')},
            'discontinuitypy.datasets_kedro': { 'discontinuitypy.datasets_kedro.E1IDsDataset': ( 'datasets_kedro.html#e1idsdataset',
                                                                                                 'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset': ( 'datasets_kedro.html#cidsdataset',
                                                                                                'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.__init__': ( 'datasets_kedro.html#cidsdataset.__init__',
                                                                                                         'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.load_data': ( 'datasets_kedro.html#cidsdataset.load_data',
                                                                                                          'discontinuitypy/datasets_kedro.py'),
                                                'discontinuitypy.datasets_kedro.cIDsDataset.load_events': ( 'datasets_kedro.html#cidsdataset.load_events',
                                                                                                            'discontinuitypy/datasets_kedro.py')},
            'discontinuitypy.detection.variance': { 'discontinuitypy.detection.variance.add_neighbor_std': ( 'detection/variance.html#add_neighbor_std',
                                                                                                             'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_combinded_std': ( 'detection/variance.html#compute_combinded_std',
                                                                                                                  'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_index_diff': ( 'detection/variance.html#compute_index_diff',
                                                                                                               'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_index_fluctuation': ( 'detection/variance.html#compute_index_fluctuation',
                                                                                                                      'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_index_std': ( 'detection/variance.html#compute_index_std',
                                                                                                              'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_indices': ( 'detection/variance.html#compute_indices',
                                                                                                            'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.compute_std': ( 'detection/variance.html#compute_std',
                                                                                                        'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.filter_indices': ( 'detection/variance.html#filter_indices',
                                                                                                           'discontinuitypy/detection/variance.py'),
                                                    'discontinuitypy.detection.variance.pl_dvec': ( 'detection/variance.html#pl_dvec',
                                                                                                    'discontinuitypy/detection/variance.py')},
            'discontinuitypy.integration': { 'discontinuitypy.integration.calc_combined_features': ( 'mag_plasma.html#calc_combined_features',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.calc_plasma_parameter_change': ( 'mag_plasma.html#calc_plasma_parameter_change',
                                                                                                           'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.calc_rotation_angle_pl': ( 'mag_plasma.html#calc_rotation_angle_pl',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.combine_features': ( 'mag_plasma.html#combine_features',
                                                                                               'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.compute_Alfven_current': ( 'mag_plasma.html#compute_alfven_current',
                                                                                                     'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.compute_inertial_length': ( 'mag_plasma.html#compute_inertial_length',
                                                                                                      'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.format_time': ( 'mag_plasma.html#format_time',
                                                                                          'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.interpolate': ( 'mag_plasma.html#interpolate',
                                                                                          'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.interpolate2': ( 'mag_plasma.html#interpolate2',
                                                                                           'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.vector_project': ( 'mag_plasma.html#vector_project',
                                                                                             'discontinuitypy/integration.py'),
                                             'discontinuitypy.integration.vector_project_pl': ( 'mag_plasma.html#vector_project_pl',
                                                                                                'discontinuitypy/integration.py')},
            'discontinuitypy.missions': { 'discontinuitypy.missions.WindConfig': ( 'mission_config.html#windconfig',
                                                                                   'discontinuitypy/missions.py'),
                                          'discontinuitypy.missions.WindMeta': ( 'mission_config.html#windmeta',
                                                                                 'discontinuitypy/missions.py')},
            'discontinuitypy.propeties.duration': { 'discontinuitypy.propeties.duration.calc_duration': ( 'properties/duration.html#calc_duration',
                                                                                                          'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.find_start_end_times': ( 'properties/duration.html#find_start_end_times',
                                                                                                                 'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.get_time_from_condition': ( 'properties/duration.html#get_time_from_condition',
                                                                                                                    'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.ts_max_derivative': ( 'properties/duration.html#ts_max_derivative',
                                                                                                              'discontinuitypy/propeties/duration.py'),
                                                    'discontinuitypy.propeties.duration.ts_max_distance': ( 'properties/duration.html#ts_max_distance',
                                                                                                            'discontinuitypy/propeties/duration.py')},
            'discontinuitypy.propeties.mva': { 'discontinuitypy.propeties.mva.calc_candidate_mva_features': ( 'properties/mva.html#calc_candidate_mva_features',
                                                                                                              'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.calc_maxiumum_variance_direction': ( 'properties/mva.html#calc_maxiumum_variance_direction',
                                                                                                                   'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.calc_mva_features': ( 'properties/mva.html#calc_mva_features',
                                                                                                    'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.fit_maxiumum_variance_direction': ( 'properties/mva.html#fit_maxiumum_variance_direction',
                                                                                                                  'discontinuitypy/propeties/mva.py'),
                                               'discontinuitypy.propeties.mva.minvar': ( 'properties/mva.html#minvar',
                                                                                         'discontinuitypy/propeties/mva.py')},
            'discontinuitypy.utils.analysis': { 'discontinuitypy.utils.analysis.filter_before_jupiter': ( 'utils/analysis_utils.html#filter_before_jupiter',
                                                                                                          'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.filter_tranges_ds': ( 'utils/analysis_utils.html#filter_tranges_ds',
                                                                                                      'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.link_coord2dim': ( 'utils/analysis_utils.html#link_coord2dim',
                                                                                                   'discontinuitypy/utils/analysis.py'),
                                                'discontinuitypy.utils.analysis.n2_normalize': ( 'utils/analysis_utils.html#n2_normalize',
                                                                                                 'discontinuitypy/utils/analysis.py')},
            'discontinuitypy.utils.basic': { 'discontinuitypy.utils.basic._expand_selectors': ( 'utils/basic.html#_expand_selectors',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.calc_vec_mag': ( 'utils/basic.html#calc_vec_mag',
                                                                                           'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.check_fgm': ( 'utils/basic.html#check_fgm',
                                                                                        'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.concat_df': ( 'utils/basic.html#concat_df',
                                                                                        'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.concat_partitions': ( 'utils/basic.html#concat_partitions',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.df2ts': ( 'utils/basic.html#df2ts',
                                                                                    'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.filter_tranges': ( 'utils/basic.html#filter_tranges',
                                                                                             'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.filter_tranges_df': ( 'utils/basic.html#filter_tranges_df',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.format_timedelta': ( 'utils/basic.html#format_timedelta',
                                                                                               'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_time': ( 'utils/basic.html#partition_data_by_time',
                                                                                                     'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_ts': ( 'utils/basic.html#partition_data_by_ts',
                                                                                                   'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_year': ( 'utils/basic.html#partition_data_by_year',
                                                                                                     'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.partition_data_by_year_month': ( 'utils/basic.html#partition_data_by_year_month',
                                                                                                           'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.pl.DataFrame.plot': ( 'utils/basic.html#pl.dataframe.plot',
                                                                                                'discontinuitypy/utils/basic.py'),
                                             'discontinuitypy.utils.basic.resample': ( 'utils/basic.html#resample',
                                                                                       'discontinuitypy/utils/basic.py')},
            'discontinuitypy.utils.plot': { 'discontinuitypy.utils.plot.plot_candidate': ( 'utils/plotting.html#plot_candidate',
                                                                                           'discontinuitypy/utils/plot.py'),
                                            'discontinuitypy.utils.plot.setup_mva_plot': ( 'utils/plotting.html#setup_mva_plot',
                                                                                           'discontinuitypy/utils/plot.py'),
                                            'discontinuitypy.utils.plot.time_stamp': ( 'utils/plotting.html#time_stamp',
                                                                                       'discontinuitypy/utils/plot.py')}}}
